# $Id: gui.py 26 2006-02-16 03:31:48Z psanchez $
#
# Copyright 2005, Pedro I. Sanchez (psanchez@colcan.biz)
#
#     This computer program is released under the
#         GNU GENERAL PUBLIC LICENSE Version 2
#              (See the file COPYING)
#

import sys

try:
    import pygtk
    pygtk.require('2.0')
    import gtk, pango
    import gtk.glade
except Exception, e:
    print e
    sys.exit(1)

import table, printing

DEFAULT_PANGO_FONT = 'Courier 10 Pitch Bold 11'

class Session(object):
    def __init__(self, gladefile):
        self.gladefile = gladefile
        self.fname = ''
        self.buff = ''
        self.page = table.Page()
        self.font = DEFAULT_PANGO_FONT
        self.table_view = None
        self.orientation = (False, True) # (horz, vert)
        self.operations = (True, False, False, False) # (+,-,x,/)

class MathTableGui(object):
    def __init__(self, session, name=None, dic={}):
        self.session = session
        self.name = name
        self.dic = dic
        self.me = None
        if self.name is not None:
            self.me = gtk.glade.XML(session.gladefile, self.name)
            self.me.signal_autoconnect(self.dic)

    def cancel(self, widget):
        self.me.get_widget(self.name).destroy()
        return True

    def push_msg(self, msg):
        self.session.statbar.push(self.session.statbar_ctx, msg)

class AboutWindow(MathTableGui):
    def __init__(self, session):
        MathTableGui.__init__(self, session,
                              name='about_window')

class TutorialWindow(MathTableGui):
    def __init__(self, session):
        dic = {'on_cancel_tutorial_clicked': self.cancel}
        MathTableGui.__init__(self, session,
                              name='tutorial_window',
                              dic=dic)

class RangesWindow(MathTableGui):
    def __init__(self, session):
        dic = {'on_spinminl_value_changed': self.minl_change,
               'on_spinmaxl_value_changed': self.maxl_change,
               'on_spinminr_value_changed': self.minr_change,
               'on_spinmaxr_value_changed': self.maxr_change,
               'on_spinacv_value_changed':  self.acv_change,
               'on_cancel_rw_clicked':      self.cancel,
               'on_ok_rw_clicked':          self.ok}
        
        MathTableGui.__init__(self, session,
                              name='ranges_window', dic=dic)
        self.minl, self.maxl, \
        self.minr, self.maxr = self.session.page.getMinMax()
        self.acv = self.session.page.getACV()
        self.spminl = self.me.get_widget('spinminl')
        self.spmaxl = self.me.get_widget('spinmaxl')
        self.spminr = self.me.get_widget('spinminr')
        self.spmaxr = self.me.get_widget('spinmaxr')
        self.spacv  = self.me.get_widget('spinacv')
        self.spminl.set_value(self.minl)
        self.spmaxl.set_value(self.maxl)
        self.spminr.set_value(self.minr)
        self.spmaxr.set_value(self.maxr)
        self.spacv.set_value(self.acv)

    def _adjustACV(self):
        """ Adjust the acv's upper limit according
            to the current operation, '+' or 'x' only.
        """
        upper = 0 # good for '-' and '/'
        if self.session.operations[0]: # '+'
            upper = self.maxl+self.maxr
        elif self.session.operations[2]: # 'x'
            upper = self.maxl*self.maxr
        adjustment = self.spacv.get_adjustment()
        adjustment.upper = upper
        self.spacv.set_adjustment(adjustment)

    def minl_change(self, widget):
        self.minl = widget.get_value_as_int()
        adjustment = self.spmaxl.get_adjustment()
        adjustment.lower = self.minl
        self.spmaxl.set_adjustment(adjustment)
        return True

    def maxl_change(self, widget):
        self.maxl = widget.get_value_as_int()
        adjustment = self.spminl.get_adjustment()
        adjustment.upper = self.maxl
        self.spminl.set_adjustment(adjustment)
        self._adjustACV()
        return True

    def minr_change(self, widget):
        self.minr = widget.get_value_as_int()
        adjustment = self.spmaxr.get_adjustment()
        adjustment.lower = self.minr
        self.spmaxr.set_adjustment(adjustment)
        return True

    def maxr_change(self, widget):
        self.maxr = widget.get_value_as_int()
        adjustment = self.spminr.get_adjustment()
        adjustment.upper = self.maxr
        self.spminr.set_adjustment(adjustment)
        self._adjustACV()
        return True

    def acv_change(self, widget):
        self.acv = widget.get_value_as_int()
        return True

    def ok(self, widget):
        self.session.page.setMinMax(self.minl,
                                    self.maxl,
                                    self.minr,
                                    self.maxr)
        self.session.page.setACV(self.acv)
        msg = 'Numbers set to Left(%s,%s) and Right(%s,%s).' % (self.minl,
                                                                self.maxl,
                                                                self.minr,
                                                                self.maxr)
        self.push_msg(msg)
        self.me.get_widget(self.name).destroy()
        return True

class SaveAsWindow(MathTableGui):
    def __init__(self, session):
        MathTableGui.__init__(self, session)
        buttons=(gtk.STOCK_CANCEL,
                 gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK,
                 gtk.RESPONSE_OK)
        dialog = gtk.FileChooserDialog(title='Save math table as ...',
                                       action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                       buttons=buttons)
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.set_do_overwrite_confirmation(True)
        dialog.set_current_name('mt.txt')
        filter = gtk.FileFilter()
        filter.set_name('All Files')
        filter.add_pattern('*')
        dialog.add_filter(filter)
        filter = gtk.FileFilter()
        filter.set_name('Text Files')
        filter.add_pattern('*.txt')
        dialog.add_filter(filter)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self.session.fname = dialog.get_filename()
            p = open(self.session.fname,'w')
            p.write(self.session.buff)
            p.close()
        dialog.destroy()

class FontWindow(MathTableGui):
    def __init__(self, session):
        MathTableGui.__init__(self, session)
        window = gtk.FontSelectionDialog("Font Selection")
        window.set_position(gtk.WIN_POS_MOUSE)
        window.ok_button.connect('clicked',
                                 self.font_selection_ok)
        window.cancel_button.connect_object('clicked',
                                            lambda wid: wid.destroy(),
                                            window)
        window.set_font_name(self.session.font)
        if not (window.flags() & gtk.VISIBLE):
            window.show()
        else:
            window.destroy()
        self.window = window

    def font_selection_ok(self, button):
        font = self.window.get_font_name()
        font_desc = pango.FontDescription(font)
        if font_desc: 
            self.session.table_view.modify_font(font_desc)
            self.session.font = font
        self.window.destroy()
        msg = 'Font set to %s' % font
        self.push_msg(msg)
        return True

class TableLayoutWindow(MathTableGui):
    def __init__(self, session):
        dic = {'on_spinlines_value_changed':     self.lines_change,
               'on_spincols_value_changed':      self.cols_change,
               'on_horizontal_toggled':          self.horizontal,
               'on_vertical_toggled':            self.vertical,
               'on_cancel_table_layout_clicked': self.cancel,
               'on_ok_table_layout_clicked':     self.ok}
        
        MathTableGui.__init__(self, session,
                              name='table_layout_window',
                              dic=dic)
        self.splines = self.me.get_widget('spinlines')
        self.spcols = self.me.get_widget('spincols')
        self.lines, self.cols = self.session.page.getLinesCols()
        self.splines.set_value(self.lines)
        self.spcols.set_value(self.cols)
        # in the following, the or_sync flag is needed because
        # the set_active operations toggle the radio buttons
        # and therefore the callback functions are called.
        # The callback function should only operate upon
        # user input.
        self.or_sync = True
        if session.orientation == (True, False):
            self.me.get_widget('horizontal').set_active(True)
            self.me.get_widget('vertical').set_active(False)
        else:
            session.orientation == (False, True)
            self.me.get_widget('horizontal').set_active(False)
            self.me.get_widget('vertical').set_active(True)
        self.or_sync = False

    def lines_change(self, widget):
        self.lines = widget.get_value_as_int()
        self.push_msg('Table set to have %s lines.' % self.lines)
        return True

    def cols_change(self, widget):
        self.cols = widget.get_value_as_int()
        self.push_msg('Table set to have %s columns per line.' % self.cols)
        return True

    def horizontal(self, widget):
        if self.or_sync: return True
        state = self.session.orientation
        if not self.session.orientation[0]:
            self.session.page.setOrientation(table.OR_HORIZONTAL)
            self.push_msg('Horizontal orientation selected.')
            self.session.orientation = (True, False)
        else:
            self.session.orientation = (False, state[1])
        return True

    def vertical(self, widget):
        if self.or_sync: return True
        state = self.session.orientation
        if not self.session.orientation[0]:
            self.session.page.setOrientation(table.OR_VERTICAL)
            self.push_msg('Vertical orientation selected.')
            self.session.orientation = (False, True)
        else:
            self.session.orientation = (state[1], False)
        return True

    def ok(self, widget):
        self.session.page.setLinesCols(self.lines, self.cols)
        self.push_msg('')
        self.me.get_widget(self.name).destroy()
        return True

class mainWindow(MathTableGui):
    def __init__(self, session):
        # If you wanted to pass
        # an argument, you would use a tuple like this:
        # dic = { "on button1_clicked" : \
        #         (self.button1_clicked, arg1,arg2) , ...
        dic = {'on_main_window_delete_event': self.delete,
               'on_main_window_destroy':      self.destroy,
               'on_print_activate':           self.print_it,
               'on_save_as_activate':         self.save_as,
               'on_quit_activate':            self.quit,
               'on_table_layout_activate':    self.table_layout,
               'on_number_ranges_activate':   self.number_ranges,
               'on_font_activate':            self.font,
               'on_tutorial_activate':        self.tutorial,
               'on_about_activate':           self.about,
               'on_addition_toggled':         self.addition,
               'on_substraction_toggled':     self.substraction,
               'on_multiplication_toggled':   self.multiplication,
               'on_division_toggled':         self.division,
               'on_runit_clicked':            self.runit,
               'on_solveit_clicked':          self.solveit
               }

        MathTableGui.__init__(self, session,
                              name='main_window', dic=dic)       
        font = pango.FontDescription(self.session.font)
        self.session.table_view = self.me.get_widget('table')
        self.session.table_buff = self.session.table_view.get_buffer()
        self.session.statbar = self.me.get_widget('statusbar')
        self.session.statbar_ctx = self.session.statbar.get_context_id('Main') 
        self.session.table_view.modify_font(font)
        return

    def delete(self, widget, event, data=None):
        return False

    def destroy(self, widget):
        gtk.main_quit()

    def print_it(self, widget, data=None):
        msg = printing.pretty_print(self.session.buff,
                                    self.session.page.getCellHeight())
        self.push_msg(msg)
        return True

    def save_as(self, widget):
        SaveAsWindow(self.session)
        return True

    def quit(self, widget):
        gtk.main_quit()

    def table_layout(self, widget):
        TableLayoutWindow(self.session)
        return True

    def number_ranges(self, widget):
        RangesWindow(self.session)
        return True

    def font(self, widget):
        FontWindow(self.session)

    def tutorial(self, widget):
        TutorialWindow(self.session)
        return True

    def about(self, widget):
        AboutWindow(self.session)
        return True

    def addition(self, widget, data=None):
        state = self.session.operations
        if not self.session.operations[0]:
            self.session.page.setOperator(table.OPER_ADDITION)
            self.push_msg('Addition selected.')
            self.session.operations = (True, False, False, False)
        else:
            self.session.operations = (False, state[1], state[2], state[3])
        return True

    def substraction(self, widget, data=None):
        state = self.session.operations
        if not self.session.operations[1]:
            self.session.page.setOperator(table.OPER_SUBSTRACTION)
            self.push_msg('Substraction selected.')
            self.session.operations = (False, True, False, False)
        else:
            self.session.operations = (state[0], False, state[2], state[3])
        return True

    def multiplication(self, widget, data=None):
        state = self.session.operations
        if not self.session.operations[2]:
            self.session.page.setOperator(table.OPER_MULTIPLICATION)
            self.push_msg('Multiplication selected.')
            self.session.operations = (False, False, True, False)
        else:
            self.session.operations = (state[0], state[1], False, state[3])
        return True

    def division(self, widget, data=None):
        state = self.session.operations
        if not self.session.operations[3]:
            self.session.page.setOperator(table.OPER_DIVISION)
            self.push_msg('Division selected.')
            self.session.operations = (False, False,  False, True)
        else:
            self.session.operations = (state[0], state[1], state[2], False)
        return True

    def runit(self, widget):
        self.session.page.answers(False)
        self.doit(widget)
        self.push_msg('New table generated (not printed yet).')
        return True

    def solveit(self, widget):
        self.session.page.answers(True)
        self.doit(widget)
        msg = 'Table with answers generated (not printed yet).'
        if len(self.session.buff) == 0:
            msg = 'No table has been generated yet!'
        self.push_msg(msg)
        return True

    def doit(self, widget):
        self.session.buff = '%s' % self.session.page
        self.session.table_buff.set_text(self.session.buff)

class mathTable:
    def __init__(self, gfile):
        self.session = Session(gfile)
        self.gui = mainWindow(self.session)

    def start(self):
        gtk.main()
